@@ -66,20 +66,37 @@ ma_r_generic <- function(es, n, var_e, sample_id = NULL, wt_type = "sample_size"
           var_e <- match_variables(call = call_full[[match("var_e",  names(call_full))]], arg = var_e, data = data)
           sample_id <- match_variables(call = call_full[[match("sample_id",  names(call_full))]], arg = sample_id, data = data)
 
-          if(deparse(substitute(sample_id)) != "NULL")
+          if(deparse(substitute(sample_id))[1] != "NULL")
                sample_id <- match_variables(call = call_full[[match("sample_id",  names(call_full))]], arg = sample_id, data = data)
 
           if(deparse(substitute(moderators))[1] != "NULL")
-               moderators <- match_variables(call = call_full[[match("moderators",  names(call_full))]], arg = moderators, data = data)
+               moderators <- match_variables(call = call_full[[match("moderators",  names(call_full))]], arg = moderators, data = as_tibble(data), as_array = TRUE)
      }
 
      if(!is.null(moderators)){
-          moderator_levels <- lapply(data.frame(data.frame(moderators)[,cat_moderators]), function(x){
+          if(is.null(dim(moderators))){
+               moderators <- as.data.frame(moderators)
+               colnames(moderators) <- "Moderator"
+          }
+
+          moderator_names <- list(all = colnames(moderators),
+                                  cat = colnames(moderators)[cat_moderators],
+                                  noncat = colnames(moderators)[!cat_moderators])
+          moderator_names <- lapply(moderator_names, function(x) if(length(x) == 0){NULL}else{x})
+
+          moderator_levels <- lapply(as_tibble(moderators)[,cat_moderators], function(x){
                lvls <- levels(x)
                if(is.null(lvls)) lvls <- levels(factor(x))
                lvls
           })
+          names(moderator_levels) <- colnames(moderators)
+
+          moderators <- as.data.frame(moderators)
      }else{
+          moderator_names <- list(all = NULL,
+                                  cat = NULL,
+                                  noncat = NULL)
+
           moderator_levels <- NULL
      }
 
@@ -97,12 +114,13 @@ ma_r_generic <- function(es, n, var_e, sample_id = NULL, wt_type = "sample_size"
 
                        ma_arg_list = list(conf_level = conf_level, cred_level = cred_level,
                                           conf_method = conf_method, cred_method = cred_method, var_unbiased = var_unbiased, wt_type = wt_type),
-                       presorted_data = additional_args$presorted_data, analysis_id_variables = additional_args$analysis_id_variables, moderator_levels = moderator_levels)
+                       presorted_data = additional_args$presorted_data, analysis_id_variables = additional_args$analysis_id_variables,
+                       moderator_levels = moderator_levels, moderator_names = moderator_names)
      out$barebones <- append(list(call = call, inputs = inputs), out$barebones)
      out <- append(list(call_history = list(call)), out)
 
      out$barebones$messages <- list(warnings = clean_warning(warn_obj1 = warn_obj1, warn_obj2 = record_warnings()),
-                                    fyi = record_fyis(neg_var_res = sum(out$barebones$meta_table$var_res < 0)))
+                                    fyi = record_fyis(neg_var_res = sum(out$barebones$meta_table$var_res < 0, na.rm = TRUE)))
 
      class(out) <- c("psychmeta", "ma_generic", "ma_bb")
      return(out)
@@ -179,8 +197,12 @@ ma_r_generic <- function(es, n, var_e, sample_id = NULL, wt_type = "sample_size"
      k <- sum(!is.na(wt_vec) & !is.na(es))
 
      if(k == 1){
+          var_es <- sd_es <- NA
+          se_es <- sd_e
           ci <- confidence(mean = mean_es, sd = sd_e, k = 1, conf_level = conf_level, conf_method = conf_method)
+          var_res <- sd_res <- NA
      }else{
+          se_es <- sd_es / sqrt(k)
           ci <- confidence(mean = mean_es, sd = sd_es, k = k, conf_level = conf_level, conf_method = conf_method)
      }
      cv <- credibility(mean = mean_es, sd = sd_res, cred_level = cred_level, k = k, cred_method = cred_method)
@@ -194,7 +216,7 @@ ma_r_generic <- function(es, n, var_e, sample_id = NULL, wt_type = "sample_size"
                                                  var_e = var_e,
                                                  var_res = var_res,
                                                  sd_es = sd_es,
-                                                 se_es = sd_es / sqrt(k),
+                                                 se_es = se_es,
                                                  sd_e = sd_e,
                                                  sd_res = sd_res,
                                                  ci, cv))),
