@@ -23,6 +23,7 @@
 #' @param rxx_restricted Logical vector or column name determining whether each element of \code{rxx} is an incumbent reliability (\code{TRUE}) or an applicant reliability (\code{FALSE}).
 #' @param ryy Vector or column name of reliability estimates for Y.
 #' @param ryy_restricted Logical vector or column name determining whether each element of \code{ryy} is an incumbent reliability (\code{TRUE}) or an applicant reliability (\code{FALSE}).
+#' @param rxx_type,ryy_type String vector identifying the types of reliability estimates supplied (e.g., "alpha", "retest", "interrater_r", "splithalf"). See the documentation for \code{\link{ma_r}} for a full list of acceptable reliability types.
 #' @param ux Vector or column name of u ratios for X.
 #' @param ux_observed Logical vector or column name determining whether each element of \code{ux} is an observed-score u ratio (\code{TRUE}) or a true-score u ratio (\code{FALSE}).
 #' @param uy Vector or column name of u ratios for Y.
@@ -38,23 +39,11 @@
 #' @param moderator_type Type of moderator analysis: "none" means that no moderators are to be used, "simple" means that moderators are to be examined one at a time, and
 #' "hierarchical" means that all possible combinations and subsets of moderators are to be examined.
 #' @param cat_moderators Logical scalar or vector identifying whether variables in the \code{moderators} argument are categorical variables (\code{TRUE}) or continuous variables (\code{FALSE}).
-#' @param impute_method Method to use for imputing artifacts. Choices are:
-#' \itemize{
-#' \item "bootstrap_mod" = select random values from the most specific moderator categories available (default).
-#' \item "bootstrap_full" = select random values from the full vector of artifacts.
-#' \item "simulate_mod" = generate random values from the distribution with the mean and variance of observed artifacts from the most specific moderator categories available.
-#' (uses \code{rnorm} for u ratios and \code{rbeta} for reliability values).
-#' \item "simulate_full" = generate random values from the distribution with the mean and variance of all observed artifacts (uses \code{rnorm} for u ratios and \code{rbeta} for reliability values).
-#' \item "wt_mean_mod" = replace missing values with the sample-size weighted mean of the distribution of artifacts from the most specific moderator categories available (not recommended).
-#' \item "wt_mean_full" = replace missing values with the sample-size weighted mean of the full distribution of artifacts (not recommended).
-#' \item "unwt_mean_mod" = replace missing values with the unweighted mean of the distribution of artifacts from the most specific moderator categories available (not recommended).
-#' \item "unwt_mean_full" = replace missing values with the unweighted mean of the full distribution of artifacts (not recommended).
-#' \item "replace_unity" = replace missing values with 1 (not recommended).
-#' \item "stop" = stop evaluations when missing artifacts are encountered.
-#' If an imputation method ending in "mod" is selected but no moderators are provided, the "mod" suffix will internally be replaced with "full".
-#' }
+#' @param impute_method Method to use for imputing artifacts. See the documentation for \code{\link{ma_r}} for a list of available imputation methods.
 #' @param hs_override When \code{TRUE}, this will override settings for \code{wt_type} (will set to "sample_size"), \code{error_type} (will set to "mean"),
 #' \code{correct_bias} (will set to \code{TRUE}), \code{conf_method} (will set to "norm"), \code{cred_method} (will set to "norm"), and \code{var_unbiased} (will set to \code{FALSE}).
+#' @param use_all_arts Logical scalar that determines whether artifact values from studies without valid effect sizes should be used in artifact distributions (\code{TRUE}) or not (\code{FALSE}).
+#' @param supplemental_ads_x,supplemental_ads_y List supplemental artifact distribution information from studies not included in the meta-analysis. The elements of this list  are named like the arguments of the \code{create_ad()} function.
 #' @param data Data frame containing columns whose names may be provided as arguments to vector arguments and/or moderators.
 #' @param ... Further arguments to be passed to functions called within the meta-analysis (e.g., create_ad_int and create_ad_tsa).
 #'
@@ -83,14 +72,15 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                     correct_bias = TRUE, correct_rxx = TRUE, correct_ryy = TRUE,
                     correct_rr_x = TRUE, correct_rr_y = TRUE,
                     indirect_rr_x = TRUE, indirect_rr_y = TRUE,
-                    rxx = NULL, rxx_restricted = TRUE,
-                    ryy = NULL, ryy_restricted = TRUE,
+                    rxx = NULL, rxx_restricted = TRUE, rxx_type = "alpha",
+                    ryy = NULL, ryy_restricted = TRUE, ryy_type = "alpha",
                     ux = NULL, ux_observed = TRUE,
                     uy = NULL, uy_observed = TRUE,
                     sign_rxz = 1, sign_ryz = 1,
                     conf_level = .95, cred_level = .8, conf_method = "t", cred_method = "t", var_unbiased = TRUE,
                     moderators = NULL, cat_moderators = TRUE, moderator_type = "simple",
-                    impute_method = "bootstrap_mod", hs_override = FALSE, data = NULL, ...){
+                    impute_method = "bootstrap_mod", hs_override = FALSE,
+                    use_all_arts = FALSE, supplemental_ads_x = NULL, supplemental_ads_y = NULL, data = NULL, ...){
 
      warn_obj1 <- record_warnings()
      call <- match.call()
@@ -118,6 +108,7 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
      sign_ryz <- scalar_arg_warning(arg = sign_ryz, arg_name = "sign_ryz")
      correct_rxx <- scalar_arg_warning(arg = correct_rxx, arg_name = "correct_rxx")
      correct_ryy <- scalar_arg_warning(arg = correct_ryy, arg_name = "correct_ryy")
+     use_all_arts <- scalar_arg_warning(arg = use_all_arts, arg_name = "use_all_arts")
 
      inputs <- list(hs_override = hs_override, wt_type = wt_type, error_type = error_type,
                     correct_bias = correct_bias, correct_rxx = correct_rxx, correct_ryy = correct_ryy,
@@ -155,7 +146,7 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
      call_full <- as.call(append(as.list(call), formal_args))
 
      if(!is.null(data)){
-          data <- data.frame(data)
+          data <- as.data.frame(data)
 
           rxyi <- match_variables(call = call_full[[match("rxyi", names(call_full))]], arg = rxyi, data = data)
           n <- match_variables(call = call_full[[match("n",  names(call_full))]], arg = n, data = data)
@@ -165,50 +156,71 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
           indirect_rr_x <- match_variables(call = call_full[[match("indirect_rr_x", names(call_full))]], arg = indirect_rr_x, data = data)
           indirect_rr_y <- match_variables(call = call_full[[match("indirect_rr_y", names(call_full))]], arg = indirect_rr_y, data = data)
 
-          if(deparse(substitute(rxx)) != "NULL")
+          if(deparse(substitute(rxx))[1] != "NULL")
                rxx <- match_variables(call = call_full[[match("rxx",  names(call_full))]], arg = rxx, data = data)
 
-          if(deparse(substitute(rxx_restricted)) != "NULL")
+          if(deparse(substitute(rxx_restricted))[1] != "NULL")
                rxx_restricted <- match_variables(call = call_full[[match("rxx_restricted", names(call_full))]], arg = rxx_restricted, data = data)
 
-          if(deparse(substitute(ryy)) != "NULL")
+          if(deparse(substitute(rxx_type))[1] != "NULL")
+               rxx_type <- match_variables(call = call_full[[match("rxx_type", names(call_full))]], arg = rxx_type, data = data)
+
+          if(deparse(substitute(ryy))[1] != "NULL")
                ryy <- match_variables(call = call_full[[match("ryy",  names(call_full))]], arg = ryy, data = data)
 
-          if(deparse(substitute(ryy_restricted)) != "NULL")
+          if(deparse(substitute(ryy_restricted))[1] != "NULL")
                ryy_restricted <- match_variables(call = call_full[[match("ryy_restricted", names(call_full))]], arg = ryy_restricted, data = data)
 
-          if(deparse(substitute(ux)) != "NULL")
+          if(deparse(substitute(ryy_type))[1] != "NULL")
+               ryy_type <- match_variables(call = call_full[[match("ryy_type", names(call_full))]], arg = ryy_type, data = data)
+
+          if(deparse(substitute(ux))[1] != "NULL")
                ux <- match_variables(call = call_full[[match("ux",  names(call_full))]], arg = ux, data = data)
 
-          if(deparse(substitute(ux_observed)) != "NULL")
+          if(deparse(substitute(ux_observed))[1] != "NULL")
                ux_observed <- match_variables(call = call_full[[match("ux_observed", names(call_full))]], arg = ux_observed, data = data)
 
-          if(deparse(substitute(uy)) != "NULL")
+          if(deparse(substitute(uy))[1] != "NULL")
                uy <- match_variables(call = call_full[[match("uy",  names(call_full))]], arg = uy, data = data)
 
-          if(deparse(substitute(uy_observed)) != "NULL")
+          if(deparse(substitute(uy_observed))[1] != "NULL")
                uy_observed <- match_variables(call = call_full[[match("uy_observed", names(call_full))]], arg = uy_observed, data = data)
 
-          if(deparse(substitute(sample_id)) != "NULL")
+          if(deparse(substitute(sample_id))[1] != "NULL")
                sample_id <- match_variables(call = call_full[[match("sample_id",  names(call_full))]], arg = sample_id, data = data)
 
           if(deparse(substitute(moderators))[1] != "NULL" & deparse(substitute(moderators)) != ".psychmeta_reserved_internal_mod_aabbccddxxyyzz")
-               moderators <- match_variables(call = call_full[[match("moderators",  names(call_full))]], arg = moderators, data = data)
+               moderators <- match_variables(call = call_full[[match("moderators",  names(call_full))]], arg = moderators, data = as_tibble(data), as_array = TRUE)
      }
 
      if(length(moderators) > 0){
-          moderator_levels <- lapply(data.frame(data.frame(moderators)[,cat_moderators]), function(x){
+          if(is.null(dim(moderators))){
+               moderators <- as.data.frame(moderators)
+               colnames(moderators) <- "Moderator"
+          }
+
+          moderator_names <- list(all = colnames(moderators),
+                                  cat = colnames(moderators)[cat_moderators],
+                                  noncat = colnames(moderators)[!cat_moderators])
+          moderator_names <- lapply(moderator_names, function(x) if(length(x) == 0){NULL}else{x})
+
+          moderator_levels <- lapply(as_tibble(moderators)[,cat_moderators], function(x){
                lvls <- levels(x)
                if(is.null(lvls)) lvls <- levels(factor(x))
                lvls
           })
+          names(moderator_levels) <- colnames(moderators)
+
+          moderators <- as.data.frame(moderators)
      }else{
+          moderator_names <- list(all = NULL,
+                                  cat = NULL,
+                                  noncat = NULL)
+
           moderator_levels <- NULL
           if(deparse(substitute(moderators)) == ".psychmeta_reserved_internal_mod_aabbccddxxyyzz") moderators <- NULL
      }
 
-
-
      ## Clear up discrepancies between arguments and feasible corrections
      if(is.null(rxx)){correct_rxx <- FALSE}else{if(all(is.na(rxx))){correct_rxx <- FALSE}}
      if(is.null(ryy)){correct_ryy <- FALSE}else{if(all(is.na(ryy))){correct_ryy <- FALSE}}
@@ -225,6 +237,48 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                warning(sum(!valid_r), " invalid correlations and/or sample sizes detected: Offending entries have been removed", call. = FALSE)
           }
 
+     rxx_type <- as.character(rxx_type)
+     ryy_type <- as.character(ryy_type)
+     rxx_type <- manage_arglength(x = rxx_type, y = rxyi)
+     ryy_type <- manage_arglength(x = ryy_type, y = rxyi)
+
+     if(use_all_arts & any(!valid_r)){
+          .rxx_type <- rxx_type[!valid_r]
+          .ryy_type <- ryy_type[!valid_r]
+
+          .n <- n[!valid_r]
+          .rxx <- manage_arglength(x = rxx, y = rxyi)[!valid_r]
+          .rxx_restricted <- manage_arglength(x = rxx_restricted, y = rxyi)[!valid_r]
+          .ryy <- manage_arglength(x = ryy, y = rxyi)[!valid_r]
+          .ryy_restricted <- manage_arglength(x = ryy_restricted, y = rxyi)[!valid_r]
+          .ux <- manage_arglength(x = ux, y = rxyi)[!valid_r]
+          .ux_observed <- manage_arglength(x = ux_observed, y = rxyi)[!valid_r]
+          .uy <- manage_arglength(x = uy, y = rxyi)[!valid_r]
+          .uy_observed <- manage_arglength(x = uy_observed, y = rxyi)[!valid_r]
+
+          .supplemental_ads <- create_ad_list(n = .n,
+                                              construct_x = rep("X", length(.n)),
+                                              construct_y = rep("Y", length(.n)),
+                                              rxx = .rxx, rxx_restricted = .rxx_restricted, rxx_type = .rxx_type,
+                                              ryy = .ryy, ryy_restricted = .ryy_restricted, ryy_type = .ryy_type,
+                                              ux = .ux, ux_observed = .ux_observed,
+                                              uy = .uy, uy_observed = .uy_observed, process_ads = FALSE)
+          .supplemental_ads_x <- .supplemental_ads$X
+          .supplemental_ads_y <- .supplemental_ads$Y
+
+          if(is.null(supplemental_ads_x)){
+               supplemental_ads_x <- .supplemental_ads_x
+          }else{
+               for(i in names(.supplemental_ads_x[[i]])) supplemental_ads_x[[i]] <- c(supplemental_ads_x[[i]], .supplemental_ads_x[[i]])
+          }
+
+          if(is.null(supplemental_ads_y)){
+               supplemental_ads_y <- .supplemental_ads_y
+          }else{
+               for(i in names(.supplemental_ads_y[[i]])) supplemental_ads_y[[i]] <- c(supplemental_ads_y[[i]], .supplemental_ads_y[[i]])
+          }
+     }
+
      ## Check the lengths of all arguments
      indirect_rr_x <- manage_arglength(x = indirect_rr_x, y = rxyi)[valid_r]
      indirect_rr_y <- manage_arglength(x = indirect_rr_y, y = rxyi)[valid_r]
@@ -237,9 +291,13 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
      uy <- manage_arglength(x = uy, y = rxyi)[valid_r]
      uy_observed <- manage_arglength(x = uy_observed, y = rxyi)[valid_r]
 
+     rxx_type <- rxx_type[valid_r]
+     ryy_type <- ryy_type[valid_r]
+
      rxyi <- rxyi[valid_r]
      n <- n[valid_r]
      n_adj <- n_adj[valid_r]
+     if(!is.null(moderators)) moderators <- data.frame(moderators)[valid_r,]
 
      ## Construct artifact distribution for X
      rxxa <-   if(!is.null(rxx)){if(any(!rxx_restricted)){rxx[!rxx_restricted]}else{NULL}}else{NULL}
@@ -251,17 +309,22 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
      ut <-     if(!is.null(ux)){if(any(!ux_observed)){ux[!ux_observed]}else{NULL}}else{NULL}
      n_ut <-   if(!is.null(ux)){if(any(!ux_observed)){n[!ux_observed]}else{NULL}}else{NULL}
 
+     rxxi_type <- if(!is.null(rxx)){if(any(rxx_restricted)){rxx_type[rxx_restricted]}else{NULL}}else{NULL}
+     rxxa_type <- if(!is.null(rxx)){if(any(!rxx_restricted)){rxx_type[!rxx_restricted]}else{NULL}}else{NULL}
+
      if(is.null(ad_x_int))
-          ad_x_int <- suppressWarnings(create_ad_int(rxxa = rxxa, wt_rxxa = n_rxxa,
-                                                     rxxi = rxxi, wt_rxxi = n_rxxi,
-                                                     ux = ux, wt_ux = n_ux,
-                                                     ut = ut, wt_ut = n_ut))
+          ad_x_int <- suppressWarnings(create_ad_supplemental(ad_type = "int", rxxa = rxxa, n_rxxa = n_rxxa, wt_rxxa = n_rxxa, rxxa_type = rxxa_type,
+                                                              rxxi = rxxi, n_rxxi = n_rxxi, wt_rxxi = n_rxxi, rxxi_type = rxxi_type,
+                                                              ux = ux, ni_ux = n_ux, wt_ux = n_ux,
+                                                              ut = ut, ni_ut = n_ut, wt_ut = n_ut,
+                                                              var_unbiased = var_unbiased, supplemental_ads = supplemental_ads_x))
 
      if(is.null(ad_x_tsa))
-          ad_x_tsa <- suppressWarnings(create_ad_tsa(rxxa = rxxa, n_rxxa = n_rxxa,
-                                                     rxxi = rxxi, n_rxxi = n_rxxi,
-                                                     ux = ux, ni_ux = n_ux,
-                                                     ut = ut, ni_ut = n_ut))
+          ad_x_tsa <- suppressWarnings(create_ad_supplemental(ad_type = "tsa", rxxa = rxxa, n_rxxa = n_rxxa, rxxa_type = rxxa_type,
+                                                              rxxi = rxxi, n_rxxi = n_rxxi, rxxi_type = rxxi_type,
+                                                              ux = ux, ni_ux = n_ux,
+                                                              ut = ut, ni_ut = n_ut,
+                                                              var_unbiased = var_unbiased, supplemental_ads = supplemental_ads_x))
 
      ## Construct artifact distribution for Y
      ryya <-   if(!is.null(ryy)){if(any(!ryy_restricted)){ryy[!ryy_restricted]}else{NULL}}else{NULL}
@@ -273,17 +336,22 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
      up <-     if(!is.null(uy)){if(any(!uy_observed)){uy[!uy_observed]}else{NULL}}else{NULL}
      n_up <-   if(!is.null(uy)){if(any(!uy_observed)){n[!uy_observed]}else{NULL}}else{NULL}
 
+     ryyi_type <- if(!is.null(ryy)){if(any(ryy_restricted)){ryy_type[ryy_restricted]}else{NULL}}else{NULL}
+     ryya_type <- if(!is.null(ryy)){if(any(!ryy_restricted)){ryy_type[!ryy_restricted]}else{NULL}}else{NULL}
+
      if(is.null(ad_y_int))
-          ad_y_int <- suppressWarnings(create_ad_int(rxxa = ryya, wt_rxxa = n_ryya,
-                                                     rxxi = ryyi, wt_rxxi = n_ryyi,
-                                                     ux = uy, wt_ux = n_uy,
-                                                     ut = up, wt_ut = n_up))
+          ad_y_int <- suppressWarnings(create_ad_supplemental(ad_type = "int", rxxa = ryya, n_rxxa = n_ryya, wt_rxxa = n_ryya, rxxa_type = ryya_type,
+                                                              rxxi = ryyi, n_rxxi = n_ryyi, wt_rxxi = n_ryyi, rxxi_type = ryyi_type,
+                                                              ux = uy, ni_ux = n_uy, wt_ux = n_uy,
+                                                              ut = up, ni_ut = n_up, wt_ut = n_up,
+                                                              var_unbiased = var_unbiased, supplemental_ads = supplemental_ads_y))
 
      if(is.null(ad_y_tsa))
-          ad_y_tsa <- suppressWarnings(create_ad_tsa(rxxa = ryya, n_rxxa = n_ryya,
-                                                     rxxi = ryyi, n_rxxi = n_ryyi,
-                                                     ux = uy, ni_ux = n_uy,
-                                                     ut = up, ni_ut = n_up))
+          ad_y_tsa <- suppressWarnings(create_ad_supplemental(ad_type = "tsa", rxxa = ryya, n_rxxa = n_ryya, rxxa_type = ryya_type,
+                                                              rxxi = ryyi, n_rxxi = n_ryyi, rxxi_type = ryyi_type,
+                                                              ux = uy, ni_ux = n_uy,
+                                                              ut = up, ni_ut = n_up,
+                                                              var_unbiased = var_unbiased, supplemental_ads = supplemental_ads_y))
 
      if(is.null(rxx)) rxx <- rep(1, length(rxyi))
      if(is.null(ryy)) ryy <- rep(1, length(rxyi))
@@ -297,15 +365,18 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
 
      if(is.null(rxx_restricted)) rxx_restricted <- rep(TRUE, length(rxyi))
      if(is.null(ryy_restricted)) ryy_restricted <- rep(TRUE, length(rxyi))
+     if(is.null(rxx_type)) rxx_type <- rep("alpha", length(rxyi))
+     if(is.null(ryy_type)) ryy_type <- rep("alpha", length(rxyi))
      if(is.null(ux_observed)) ux_observed <- rep(TRUE, length(rxyi))
      if(is.null(uy_observed)) uy_observed <- rep(TRUE, length(rxyi))
 
      if(all(is.na(rxx_restricted))) rxx_restricted <- rep(TRUE, length(rxyi))
      if(all(is.na(ryy_restricted))) ryy_restricted <- rep(TRUE, length(rxyi))
+     if(all(is.na(rxx_type))) rxx_type <- rep("alpha", length(rxyi))
+     if(all(is.na(ryy_type))) ryy_type <- rep("alpha", length(rxyi))
      if(all(is.na(ux_observed))) ux_observed <- rep(TRUE, length(rxyi))
      if(all(is.na(uy_observed))) uy_observed <- rep(TRUE, length(rxyi))
 
-
      if(correct_rxx & any(!is.na(rxx))) screen_rel(rel_vec = rxx[!is.na(rxx)], art_name = "rxx")
      if(correct_ryy & any(!is.na(ryy))) screen_rel(rel_vec = ryy[!is.na(ryy)], art_name = "ryy")
 
@@ -431,7 +502,7 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                ## Estimate the necessary incumbent relibilities for X
                ## If the u ratio for X is known:
                subset_vec1 <- valid_rxxa & !rxx_restricted & rr_eligible_x & (do_uvirr_x | do_uvdrr_y | do_uvirr_y)
-               rxxi_vec[subset_vec1] <- estimate_rxxi(rxxa = rxx[subset_vec1], ux = ux[subset_vec1], ux_observed = ux_observed[subset_vec1], indirect_rr = indirect_rr_x[subset_vec1])
+               rxxi_vec[subset_vec1] <- estimate_rxxi(rxxa = rxx[subset_vec1], ux = ux[subset_vec1], ux_observed = ux_observed[subset_vec1], indirect_rr = indirect_rr_x[subset_vec1], rxxa_type = rxx_type[subset_vec1])
 
                ## If the u ratio for X is unknown, but the u ratio for Y is known:
                subset_vec2 <- valid_rxxa & !rxx_restricted & !rr_eligible_x & rr_eligible_y & (do_uvdrr_y | do_uvirr_y)
@@ -457,7 +528,7 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                ## Estimate the necessary applicant relibilities for X
                ## If the u ratio for X is known:
                subset_vec <- valid_rxxi & rxx_restricted & rr_eligible_x & (do_uvdrr_x | do_uvirr_x | do_bvirr | do_bvdrr)
-               rxxa_vec[subset_vec] <- estimate_rxxa(rxxi = rxx[subset_vec], ux = ux[subset_vec], ux_observed = ux_observed[subset_vec], indirect_rr = indirect_rr_x[subset_vec])
+               rxxa_vec[subset_vec] <- estimate_rxxa(rxxi = rxx[subset_vec], ux = ux[subset_vec], ux_observed = ux_observed[subset_vec], indirect_rr = indirect_rr_x[subset_vec], rxxi_type = rxx_type[subset_vec])
                subset_vec <- (do_uvdrr_x | do_uvirr_x | do_bvirr | do_bvdrr)
                rxxa_vec[!subset_vec] <- NA
 
@@ -484,7 +555,7 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                ## Estimate the necessary incumbent relibilities for Y
                ## If the u ratio for Y is known:
                subset_vec1 <- valid_ryya & !ryy_restricted & rr_eligible_y & (do_uvirr_y | do_uvdrr_x | do_uvirr_x)
-               ryyi_vec[subset_vec1] <- estimate_rxxi(rxxa = ryy[subset_vec1], ux = uy[subset_vec1], ux_observed = uy_observed[subset_vec1], indirect_rr = indirect_rr_y[subset_vec1])
+               ryyi_vec[subset_vec1] <- estimate_rxxi(rxxa = ryy[subset_vec1], ux = uy[subset_vec1], ux_observed = uy_observed[subset_vec1], indirect_rr = indirect_rr_y[subset_vec1], rxxa_type = ryy_type[subset_vec1])
 
                ## If the u ratio for Y is unknown, but the u ratio for X is known:
                subset_vec2 <- valid_ryya & !ryy_restricted & !rr_eligible_y & rr_eligible_x & (do_uvdrr_x | do_uvirr_x)
@@ -510,7 +581,7 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                ## Estimate the necessary applicant relibilities for Y
                ## If the u ratio for Y is known:
                subset_vec <- valid_ryyi & ryy_restricted & rr_eligible_y & (do_uvdrr_y | do_uvirr_y | do_bvirr | do_bvdrr)
-               ryya_vec[subset_vec] <- estimate_rxxa(rxxi = ryy[subset_vec], ux = uy[subset_vec], ux_observed = uy_observed[subset_vec], indirect_rr = indirect_rr_y[subset_vec])
+               ryya_vec[subset_vec] <- estimate_rxxa(rxxi = ryy[subset_vec], ux = uy[subset_vec], ux_observed = uy_observed[subset_vec], indirect_rr = indirect_rr_y[subset_vec], rxxi_type = rxx_type[subset_vec])
                subset_vec <- (do_uvdrr_y | do_uvirr_y | do_bvirr | do_bvdrr)
                ryya_vec[!subset_vec] <- NA
 
@@ -817,17 +888,18 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                        ma_arg_list = list(error_type = error_type, correct_bias = correct_bias, conf_level = conf_level, cred_level = cred_level,
                                           conf_method = conf_method, cred_method = cred_method, var_unbiased = var_unbiased, wt_type = wt_type,
                                           sign_rxz = sign_rxz, sign_ryz = sign_ryz, es_d = es_d, treat_as_d = treat_as_d),
-                       presorted_data = additional_args$presorted_data, analysis_id_variables = additional_args$analysis_id_variables, moderator_levels = moderator_levels)
+                       presorted_data = additional_args$presorted_data, analysis_id_variables = additional_args$analysis_id_variables,
+                       moderator_levels = moderator_levels, moderator_names = moderator_names)
 
      out$barebones <- append(list(call = call, inputs = inputs), out$barebones)
      out$individual_correction <- append(list(call = call, inputs = inputs, correction_data = correction_data), out$individual_correction)
      out <- append(list(call_history = list(call)), out)
      out$individual_correction$artifact_distributions <- list(ad_x_int = ad_x_int, ad_x_tsa = ad_x_tsa, ad_y_int = ad_y_int, ad_y_tsa = ad_y_tsa)
 
-     neg_var_res <- sum(out$barebones$meta_table$var_res < 0)
-     neg_var_rtpa <- sum(out$individual_correction$true_score$meta_table$var_rho < 0)
-     neg_var_rxpa <- sum(out$individual_correction$validity_generalization_x$meta_table$var_rho < 0)
-     neg_var_rtya <- sum(out$individual_correction$validity_generalization_y$meta_table$var_rho < 0)
+     neg_var_res <- sum(out$barebones$meta_table$var_res < 0, na.rm = TRUE)
+     neg_var_rtpa <- sum(out$individual_correction$true_score$meta_table$var_rho < 0, na.rm = TRUE)
+     neg_var_rxpa <- sum(out$individual_correction$validity_generalization_x$meta_table$var_rho < 0, na.rm = TRUE)
+     neg_var_rtya <- sum(out$individual_correction$validity_generalization_y$meta_table$var_rho < 0, na.rm = TRUE)
 
      out$barebones$messages <- list(warnings = NULL,
                                     fyi = record_fyis(neg_var_res = neg_var_res))
@@ -922,20 +994,25 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
           var_rtpa <- wt_var(x = rtpa_vec, wt = wt_vec_tp, unbiased = var_unbiased)
           var_e_tp_a <- wt_mean(x = var_e_tp_vec, wt = wt_vec_tp)
           var_rho_tp_a <- var_rtpa - var_e_tp_a
-          if(k == 1){
-               ci_tp_a <- confidence(mean = mean_rtpa, sd = var_e_tp_a^.5, k = 1, conf_level = conf_level, conf_method = conf_method)
-          }else{
-               ci_tp_a <- confidence(mean = mean_rtpa, sd = var_rtpa^.5, k = k, conf_level = conf_level, conf_method = conf_method)
-          }
-          cv_tp_a <- credibility(mean = mean_rtpa, sd = var_rho_tp_a^.5, cred_level = cred_level, k = k, cred_method = cred_method)
-          ci_tp_a <- setNames(c(ci_tp_a), colnames(ci_tp_a))
-          cv_tp_a <- setNames(c(cv_tp_a), colnames(cv_tp_a))
 
           sd_rtpa <- var_rtpa^.5
           sd_e_tp_a <- var_e_tp_a^.5
           sd_rho_tp_a <- var_rho_tp_a^.5
           sd_rho_tp_a[is.na(sd_rho_tp_a)] <- 0
 
+          if(k == 1){
+               var_rtpa <- sd_rtpa <- NA
+               se_rtpa <- sd_e_tp_a
+               ci_tp_a <- confidence(mean = mean_rtpa, sd = var_e_tp_a^.5, k = 1, conf_level = conf_level, conf_method = conf_method)
+               var_rho_tp_a <- sd_rho_tp_a <- NA
+          }else{
+               se_rtpa <- sd_rtpa / sqrt(k)
+               ci_tp_a <- confidence(mean = mean_rtpa, sd = var_rtpa^.5, k = k, conf_level = conf_level, conf_method = conf_method)
+          }
+          cv_tp_a <- credibility(mean = mean_rtpa, sd = var_rho_tp_a^.5, cred_level = cred_level, k = k, cred_method = cred_method)
+          ci_tp_a <- setNames(c(ci_tp_a), colnames(ci_tp_a))
+          cv_tp_a <- setNames(c(cv_tp_a), colnames(cv_tp_a))
+
           if(run_lean){
                escalc_tp <- NULL
           }else{
@@ -961,7 +1038,7 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                                                                              var_e_c = var_e_tp_a,
                                                                              var_rho = var_rho_tp_a,
                                                                              sd_r_c = sd_rtpa,
-                                                                             se_r_c = sd_rtpa / sqrt(k),
+                                                                             se_r_c = se_rtpa,
                                                                              sd_e_c = sd_e_tp_a,
                                                                              sd_rho = sd_rho_tp_a,
                                                                              ci_tp_a, cv_tp_a))),
@@ -984,20 +1061,25 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
           var_rxpa <- wt_var(x = rxpa_vec, wt = wt_vec_xp, unbiased = var_unbiased)
           var_e_xp_a <- wt_mean(x = var_e_xp_vec, wt = wt_vec_xp)
           var_rho_xp_a <- var_rxpa - var_e_xp_a
-          if(k == 1){
-               ci_xp_a <- confidence(mean = mean_rxpa, sd = var_e_xp_a^.5, k = 1, conf_level = conf_level, conf_method = conf_method)
-          }else{
-               ci_xp_a <- confidence(mean = mean_rxpa, sd = var_rxpa^.5, k = k, conf_level = conf_level, conf_method = conf_method)
-          }
-          cv_xp_a <- credibility(mean = mean_rxpa, sd = var_rho_xp_a^.5, cred_level = cred_level, k = k, cred_method = cred_method)
-          ci_xp_a <- setNames(c(ci_xp_a), colnames(ci_xp_a))
-          cv_xp_a <- setNames(c(cv_xp_a), colnames(cv_xp_a))
 
           sd_rxpa <- var_rxpa^.5
           sd_e_xp_a <- var_e_xp_a^.5
           sd_rho_xp_a <- var_rho_xp_a^.5
           sd_rho_xp_a[is.na(sd_rho_xp_a)] <- 0
 
+          if(k == 1){
+               var_rxpa <- sd_rxpa <- NA
+               se_rxpa <- sd_e_xp_a
+               ci_xp_a <- confidence(mean = mean_rxpa, sd = var_e_xp_a^.5, k = 1, conf_level = conf_level, conf_method = conf_method)
+               var_rho_xp_a <- sd_rho_xp_a <- NA
+          }else{
+               se_rxpa <- sd_rxpa / sqrt(k)
+               ci_xp_a <- confidence(mean = mean_rxpa, sd = var_rxpa^.5, k = k, conf_level = conf_level, conf_method = conf_method)
+          }
+          cv_xp_a <- credibility(mean = mean_rxpa, sd = var_rho_xp_a^.5, cred_level = cred_level, k = k, cred_method = cred_method)
+          ci_xp_a <- setNames(c(ci_xp_a), colnames(ci_xp_a))
+          cv_xp_a <- setNames(c(cv_xp_a), colnames(cv_xp_a))
+
           if(run_lean){
                escalc_xp <- NULL
           }else{
@@ -1023,7 +1105,7 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                                                                                             var_e_c = var_e_xp_a,
                                                                                             var_rho = var_rho_xp_a,
                                                                                             sd_r_c = sd_rxpa,
-                                                                                            se_r_c = sd_rxpa / sqrt(k),
+                                                                                            se_r_c = se_rxpa,
                                                                                             sd_e_c = sd_e_xp_a,
                                                                                             sd_rho = sd_rho_xp_a,
                                                                                             ci_xp_a, cv_xp_a))),
@@ -1045,20 +1127,25 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
           var_rtya <- wt_var(x = rtya_vec, wt = wt_vec_ty, unbiased = var_unbiased)
           var_e_ty_a <- wt_mean(x = var_e_ty_vec, wt = wt_vec_ty)
           var_rho_ty_a <- var_rtya - var_e_ty_a
-          if(k == 1){
-               ci_ty_a <- confidence(mean = mean_rtya, sd = var_e_ty_a^.5, k = 1, conf_level = conf_level, conf_method = conf_method)
-          }else{
-               ci_ty_a <- confidence(mean = mean_rtya, sd = var_rtya^.5, k = k, conf_level = conf_level, conf_method = conf_method)
-          }
-          cv_ty_a <- credibility(mean = mean_rtya, sd = var_rho_ty_a^.5, cred_level = cred_level, k = k, cred_method = cred_method)
-          ci_ty_a <- setNames(c(ci_ty_a), colnames(ci_ty_a))
-          cv_ty_a <- setNames(c(cv_ty_a), colnames(cv_ty_a))
 
           sd_rtya <- var_rtya^.5
           sd_e_ty_a <- var_e_ty_a^.5
           sd_rho_ty_a <- var_rho_ty_a^.5
           sd_rho_ty_a[is.na(sd_rho_ty_a)] <- 0
 
+          if(k == 1){
+               var_rtya <- sd_rtya <- NA
+               se_rtya <- sd_e_ty_a
+               ci_ty_a <- confidence(mean = mean_rtya, sd = var_e_ty_a^.5, k = 1, conf_level = conf_level, conf_method = conf_method)
+               var_rho_ty_a <- sd_rho_ty_a <- NA
+          }else{
+               se_rtya <- sd_rtya / sqrt(k)
+               ci_ty_a <- confidence(mean = mean_rtya, sd = var_rtya^.5, k = k, conf_level = conf_level, conf_method = conf_method)
+          }
+          cv_ty_a <- credibility(mean = mean_rtya, sd = var_rho_ty_a^.5, cred_level = cred_level, k = k, cred_method = cred_method)
+          ci_ty_a <- setNames(c(ci_ty_a), colnames(ci_ty_a))
+          cv_ty_a <- setNames(c(cv_ty_a), colnames(cv_ty_a))
+
           if(run_lean){
                escalc_ty <- NULL
           }else{
@@ -1083,7 +1170,7 @@ ma_r_ic <- function(rxyi, n, n_adj = NULL, sample_id = NULL,
                                                                                             var_e_c = var_e_ty_a,
                                                                                             var_rho = var_rho_ty_a,
                                                                                             sd_r_c = sd_rtya,
-                                                                                            se_r_c = sd_rtya / sqrt(k),
+                                                                                            se_r_c = se_rtya,
                                                                                             sd_e_c = sd_e_ty_a,
                                                                                             sd_rho = sd_rho_ty_a,
                                                                                             ci_ty_a, cv_ty_a))),
