@@ -10,9 +10,13 @@ sensitivity_leave1out <- function(ma_obj, ...){
 
      if(is.null(es_type)) stop("ma_obj must represent a meta-analysis of correlations or d values", call. = FALSE)
 
-     d_metric <- ifelse(any(class_ma == "ma_d_as_d" | class_ma == "ma_r_as_d"), TRUE, FALSE)
-
-     if(d_metric) ma_obj <- convert_ma(ma_obj)
+     d_metric <- ifelse(any((class_ma == "ma_d_as_d" & (any(class_ma == "ma_ic") | any(class_ma == "ma_ad"))) | class_ma == "ma_r_as_d"), TRUE, FALSE)
+     if(d_metric){
+          ma_obj <- convert_ma(ma_obj)
+          convert_back <- TRUE
+     }else{
+          convert_back <- FALSE
+     }
 
      if(any(class(ma_obj) == "ma_master")){
           ma_list <- ma_obj$construct_pairs
@@ -27,7 +31,12 @@ sensitivity_leave1out <- function(ma_obj, ...){
           record_call <- TRUE
      }
 
+     progbar <- progress_bar$new(format = " Computing leave-one-out meta-analyses [:bar] :percent est. time remaining: :eta",
+                                 total = sum(unlist(lapply(ma_list, function(x){nrow(x$barebones$meta_table)}))),
+                                 clear = FALSE, width = options()$width)
      ma_list <- lapply(ma_list, function(ma_obj_i){
+          progbar$tick()
+
           if(any(class(ma_obj_i) == "ma_ic")){
                ma_arg_list <- ma_obj_i$individual_correction$inputs
           }else{
@@ -116,16 +125,17 @@ sensitivity_leave1out <- function(ma_obj, ...){
 
           for(i in 1:k_analyses){
                analysis_id <- paste0("Analysis ID = ", i)
+               progbar$tick()
 
-               if(!is.null(ma_obj$barebones$escalc_list[[i]]$pi)){
-                    p <- wt_mean(x = ma_obj$barebones$escalc_list[[i]]$pi, wt = ma_obj$barebones$escalc_list[[i]]$n_adj)
+               if(!is.null(ma_obj_i$barebones$escalc_list[[i]]$pi)){
+                    p <- wt_mean(x = ma_obj_i$barebones$escalc_list[[i]]$pi, wt = ma_obj_i$barebones$escalc_list[[i]]$n_adj)
                }else{
                     p <- .5
                }
-               conf_level <- ma_obj$barebones$inputs$conf_level
-               cred_level <- ma_obj$barebones$inputs$cred_level
-               conf_method <- ma_obj$barebones$inputs$conf_method
-               cred_method <- ma_obj$barebones$inputs$cred_method
+               conf_level <- ma_obj_i$barebones$inputs$conf_level
+               cred_level <- ma_obj_i$barebones$inputs$cred_level
+               conf_method <- ma_obj_i$barebones$inputs$conf_method
+               cred_method <- ma_obj_i$barebones$inputs$cred_method
 
                if(es_type == "es"){
                     es_data <- data.frame(yi = yi[[i]],
@@ -168,12 +178,12 @@ sensitivity_leave1out <- function(ma_obj, ...){
                          vgx_table <- rep_list$validity_generalization_x
                          vgy_table <- rep_list$validity_generalization_y
 
-                         bb_mat <- ma_obj$barebones$meta_table[i,]
-                         ts_mat <- ma_obj$individual_correction$true_score$meta_table[i,]
-                         vgx_mat <- ma_obj$individual_correction$validity_generalization_x$meta_table[i,]
-                         vgy_mat <- ma_obj$individual_correction$validity_generalization_y$meta_table[i,]
+                         bb_mat <- ma_obj_i$barebones$meta_table[i,]
+                         ts_mat <- ma_obj_i$individual_correction$true_score$meta_table[i,]
+                         vgx_mat <- ma_obj_i$individual_correction$validity_generalization_x$meta_table[i,]
+                         vgy_mat <- ma_obj_i$individual_correction$validity_generalization_y$meta_table[i,]
 
-                         if(d_metric){
+                         if(convert_back){
                               bb_mat <- .convert_ma(ma_table = bb_mat, p_vec = rep(p, nrow(bb_mat)), conf_level = conf_level, cred_level = cred_level,
                                                     conf_method = conf_method, cred_method = cred_method)
                               ts_mat <- .convert_ma(ma_table = ts_mat, p_vec = rep(p, nrow(ts_mat)), conf_level = conf_level, cred_level = cred_level,
@@ -207,7 +217,7 @@ sensitivity_leave1out <- function(ma_obj, ...){
                     }
 
                     if(any(class_ma == "ma_ad")){
-                         ma_ad_dump_full <- do.call(.ma_r_ad, append(ma_obj$artifact_distribution$inputs, list(.psychmeta_internal_request_datadump = TRUE)))
+                         ma_ad_dump_full <- do.call(.ma_r_ad, append(ma_obj_i$artifact_distribution$inputs, list(.psychmeta_internal_request_datadump = TRUE)))
                          ma_ad_dump <- ma_ad_dump_full$x
                          ma_ad_dump$art_grid <- ma_ad_dump_full$art_grid
                          ma_arg_list$ma_ad_dump <- ma_ad_dump
@@ -219,12 +229,12 @@ sensitivity_leave1out <- function(ma_obj, ...){
                          vgx_table <- rep_list$validity_generalization_x
                          vgy_table <- rep_list$validity_generalization_y
 
-                         bb_mat <- ma_obj$barebones$meta_table[i,]
-                         ts_mat <- ma_obj$artifact_distribution$true_score$meta_table[i,]
-                         vgx_mat <- ma_obj$artifact_distribution$validity_generalization_x$meta_table[i,]
-                         vgy_mat <- ma_obj$artifact_distribution$validity_generalization_y$meta_table[i,]
+                         bb_mat <- ma_obj_i$barebones$meta_table[i,]
+                         ts_mat <- ma_obj_i$artifact_distribution$true_score$meta_table[i,]
+                         vgx_mat <- ma_obj_i$artifact_distribution$validity_generalization_x$meta_table[i,]
+                         vgy_mat <- ma_obj_i$artifact_distribution$validity_generalization_y$meta_table[i,]
 
-                         if(d_metric){
+                         if(convert_back){
                               bb_mat <- .convert_ma(ma_table = bb_mat, p_vec = rep(p, nrow(bb_mat)), conf_level = conf_level, cred_level = cred_level,
                                                       conf_method = conf_method, cred_method = cred_method)
                               ts_mat <- .convert_ma(ma_table = ts_mat, p_vec = rep(p, nrow(ts_mat)), conf_level = conf_level, cred_level = cred_level,
@@ -259,7 +269,7 @@ sensitivity_leave1out <- function(ma_obj, ...){
                }else{
                     if(any(class_ma == "ma_bb")){
 
-                         bb_mat <- ma_obj$barebones$meta_table[i,]
+                         bb_mat <- ma_obj_i$barebones$meta_table[i,]
 
                          if(es_type == "es"){
                               es_data$vi <- vi_xy[[i]]
@@ -272,7 +282,7 @@ sensitivity_leave1out <- function(ma_obj, ...){
                               es_data$weight <- wt_xy[[i]]
                               bb_table <- .ma_leave1out(data = es_data, ma_fun_boot = .ma_r_bb_boot, ma_arg_list = ma_arg_list)
 
-                              if(d_metric){
+                              if(convert_back){
                                    bb_mat <- .convert_ma(ma_table = bb_mat, p_vec = rep(p, nrow(bb_mat)), conf_level = conf_level, cred_level = cred_level,
                                                          conf_method = conf_method, cred_method = cred_method)
                                    bb_table <- .convert_ma(ma_table = bb_table, p_vec = rep(p, nrow(bb_mat)), conf_level = conf_level, cred_level = cred_level,
@@ -283,9 +293,9 @@ sensitivity_leave1out <- function(ma_obj, ...){
                          if(es_type == "d"){
                               es_data$vi <- vi[[i]]
                               es_data$weight <- wt[[i]]
-                              bb_table <- .ma_leave1out(data = es_data, ma_fun_boot = .ma_r_bb_boot, ma_arg_list = ma_arg_list)
+                              bb_table <- .ma_leave1out(data = es_data, ma_fun_boot = .ma_d_bb_boot, ma_arg_list = ma_arg_list)
 
-                              if(!d_metric){
+                              if(convert_back){
                                    bb_mat <- .convert_ma(ma_table = bb_mat, p_vec = rep(p, nrow(bb_mat)), conf_level = conf_level, cred_level = cred_level,
                                                          conf_method = conf_method, cred_method = cred_method)
                                    bb_table <- .convert_ma(ma_table = bb_table, p_vec = rep(p, nrow(bb_mat)), conf_level = conf_level, cred_level = cred_level,
@@ -309,7 +319,7 @@ sensitivity_leave1out <- function(ma_obj, ...){
           ma_obj <- ma_list[[1]]
      }
 
-     if(d_metric) ma_obj <- convert_ma(ma_obj)
+     if(convert_back) ma_obj <- convert_ma(ma_obj)
 
      if(record_call) ma_obj$call_history <- append(ma_obj$call_history, list(match.call()))
 
